@@ -26,21 +26,34 @@ if(xsystem("which cdclconfig"))
   $LOCAL_LIBS += ' ' + ldflags
   $LOCAL_LIBS += ldlibs
 
-  ldlibs = `cdclconfig --ldlibs`
-  if ldlibs =~ /-lgtk/ then
-    print "**Infomation** compile with gtk+\n"
+  gtkversion = `cdclconfig --gtkversion`.chop
+  if $?.class==Fixnum
+    status = $?
+  else
+    status = $?.exitstatus
+  end
+  if status != 0
+    if ldlibs =~ /gtk/
+      gtkversion = "1"
+    else
+      gtkversion = "no"
+    end
+  end
+  if gtkversion == "no" then
+    File.copy("init.c.default","init.c")
+  else
+    print "**Infomation** compile with gtk+ (ver.#{gtkversion})\n"
     File.copy("grph1_zgpack.c.org","grph1_zgpack.c")
     File.safe_unlink("init.c")
     File.copy("init.c.gtk","init.c")
     if ldlibs =~ /-lgdk_imlib/ then
       print "**Infomation** compile with imlib_gdk\n"
-    end    
-  else
-    File.copy("init.c.default","init.c")
+    end
   end
 
 else
   print "**Warning** cdclconfig is not found. Assume C version of DCL 5.2\n"
+  gtkversion="no"
   dcllibname = "cdcl52"
   f2clibname = "f2c4dcl"
   xlibname = "X11"
@@ -60,6 +73,9 @@ end
 
 if /cygwin|mingw/ =~ RUBY_PLATFORM
    have_library("narray") || raise("ERROR: narray library is not found")
+   if gtkversion == "2"
+     $LOCAL_LIBS += "- lruby-glib2"
+   end
 end
 
 create_makefile("numru/dcl_raw")
