@@ -73,8 +73,8 @@ dcl_shtint(obj, mm, jm, im)
     integer i_mm;
     integer i_jm;
     integer i_im;
-    real *o_q;
-    VALUE q;
+    real *o_work;
+    VALUE work;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
       mm = rb_funcall(mm, rb_intern("to_i"), 0);
@@ -90,16 +90,16 @@ dcl_shtint(obj, mm, jm, im)
     i_jm = NUM2INT(jm);
     i_im = NUM2INT(im);
 
-    o_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
+    o_work= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtint_(&i_mm, &i_jm, &i_im, o_q);
+    shtint_(&i_mm, &i_jm, &i_im, o_work);
 
     {int array_shape[1] = {(i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15};
-     q = dcl_crealary2obj(o_q, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15, 1, array_shape);
+     work = dcl_crealary2obj(o_work, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15, 1, array_shape);
     }
 
 
-    return q;
+    return work;
 
 }
 
@@ -183,14 +183,14 @@ dcl_shtnml(obj, mm, n, m)
 }
 
 static VALUE
-dcl_shtfun(obj, mm, jm, m)
-    VALUE obj, mm, jm, m;
+dcl_shtfun(obj, mm, jm, m, work)
+    VALUE obj, mm, jm, m, work;
 {
     integer i_mm;
     integer i_jm;
     integer i_m;
     real *o_fun;
-    real *w_q;
+    real *i_work;
     VALUE fun;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -202,28 +202,36 @@ dcl_shtfun(obj, mm, jm, m)
     if ((TYPE(m) != T_BIGNUM) || (TYPE(m) != T_FIXNUM)) {
       m = rb_funcall(m, rb_intern("to_i"), 0);
     }
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_m = NUM2INT(m);
+    i_work = dcl_obj2crealary(work);
 
     o_fun= ALLOCA_N(real, (2*i_jm+1)*(i_mm-i_m+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtfun_(&i_mm, &i_jm, &i_m, o_fun, w_q);
+    shtfun_(&i_mm, &i_jm, &i_m, o_fun, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(i_mm-i_m+1)};
      fun = dcl_crealary2obj(o_fun, (2*i_jm+1)*(i_mm-i_m+1), 1, array_shape);
     }
 
+    dcl_freecrealary(i_work);
 
     return fun;
 
 }
 
 static VALUE
-dcl_shtlfw(obj, mm, jm, m, isw, wm)
-    VALUE obj, mm, jm, m, isw, wm;
+dcl_shtlfw(obj, mm, jm, m, isw, wm, work)
+    VALUE obj, mm, jm, m, isw, wm, work;
 {
     integer i_mm;
     integer i_jm;
@@ -231,7 +239,7 @@ dcl_shtlfw(obj, mm, jm, m, isw, wm)
     integer i_isw;
     real *i_wm;
     real *o_sm;
-    real *w_q;
+    real *i_work;
     VALUE sm;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -253,31 +261,39 @@ dcl_shtlfw(obj, mm, jm, m, isw, wm)
            (rb_obj_is_kind_of(wm, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_m = NUM2INT(m);
     i_isw = NUM2INT(isw);
     i_wm = dcl_obj2crealary(wm);
+    i_work = dcl_obj2crealary(work);
 
     o_sm= ALLOCA_N(real, (i_mm-i_m+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtlfw_(&i_mm, &i_jm, &i_m, &i_isw, i_wm, o_sm, w_q);
+    shtlfw_(&i_mm, &i_jm, &i_m, &i_isw, i_wm, o_sm, i_work);
 
     {int array_shape[1] = {i_mm-i_m+1};
      sm = dcl_crealary2obj(o_sm, (i_mm-i_m+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_wm);
+    dcl_freecrealary(i_work);
 
     return sm;
 
 }
 
 static VALUE
-dcl_shtlbw(obj, mm, jm, m, isw, sm)
-    VALUE obj, mm, jm, m, isw, sm;
+dcl_shtlbw(obj, mm, jm, m, isw, sm, work)
+    VALUE obj, mm, jm, m, isw, sm, work;
 {
     integer i_mm;
     integer i_jm;
@@ -285,7 +301,7 @@ dcl_shtlbw(obj, mm, jm, m, isw, sm)
     integer i_isw;
     real *i_sm;
     real *o_wm;
-    real *w_q;
+    real *i_work;
     VALUE wm;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -307,38 +323,46 @@ dcl_shtlbw(obj, mm, jm, m, isw, sm)
            (rb_obj_is_kind_of(sm, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_m = NUM2INT(m);
     i_isw = NUM2INT(isw);
     i_sm = dcl_obj2crealary(sm);
+    i_work = dcl_obj2crealary(work);
 
     o_wm= ALLOCA_N(real, (2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtlbw_(&i_mm, &i_jm, &i_m, &i_isw, i_sm, o_wm, w_q);
+    shtlbw_(&i_mm, &i_jm, &i_m, &i_isw, i_sm, o_wm, i_work);
 
     {int array_shape[1] = {2*i_jm+1};
      wm = dcl_crealary2obj(o_wm, (2*i_jm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_sm);
+    dcl_freecrealary(i_work);
 
     return wm;
 
 }
 
 static VALUE
-dcl_shts2w(obj, mm, jm, isw, s)
-    VALUE obj, mm, jm, isw, s;
+dcl_shts2w(obj, mm, jm, isw, s, work)
+    VALUE obj, mm, jm, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
     integer i_isw;
     real *i_s;
     real *o_w;
-    real *w_q;
+    real *i_work;
     VALUE w;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -357,30 +381,38 @@ dcl_shts2w(obj, mm, jm, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (2*i_jm+1)*(2*i_mm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shts2w_(&i_mm, &i_jm, &i_isw, i_s, o_w, w_q);
+    shts2w_(&i_mm, &i_jm, &i_isw, i_s, o_w, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(2*i_mm+1)};
      w = dcl_crealary2obj(o_w, (2*i_jm+1)*(2*i_mm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return w;
 
 }
 
 static VALUE
-dcl_shtswa(obj, mm, jm, isw, m1, m2, s)
-    VALUE obj, mm, jm, isw, m1, m2, s;
+dcl_shtswa(obj, mm, jm, isw, m1, m2, s, work)
+    VALUE obj, mm, jm, isw, m1, m2, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -389,7 +421,7 @@ dcl_shtswa(obj, mm, jm, isw, m1, m2, s)
     integer i_m2;
     real *i_s;
     real *o_w;
-    real *w_q;
+    real *i_work;
     VALUE w;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -414,6 +446,13 @@ dcl_shtswa(obj, mm, jm, isw, m1, m2, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -421,32 +460,33 @@ dcl_shtswa(obj, mm, jm, isw, m1, m2, s)
     i_m1 = NUM2INT(m1);
     i_m2 = NUM2INT(m2);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (2*i_jm+1)*(2*i_mm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtswa_(&i_mm, &i_jm, &i_isw, &i_m1, &i_m2, i_s, o_w, w_q);
+    shtswa_(&i_mm, &i_jm, &i_isw, &i_m1, &i_m2, i_s, o_w, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(2*i_mm+1)};
      w = dcl_crealary2obj(o_w, (2*i_jm+1)*(2*i_mm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return w;
 
 }
 
 static VALUE
-dcl_shtswz(obj, mm, jm, isw, s)
-    VALUE obj, mm, jm, isw, s;
+dcl_shtswz(obj, mm, jm, isw, s, work)
+    VALUE obj, mm, jm, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
     integer i_isw;
     real *i_s;
     real *o_wz;
-    real *w_q;
+    real *i_work;
     VALUE wz;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -465,30 +505,38 @@ dcl_shtswz(obj, mm, jm, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_wz= ALLOCA_N(real, (2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtswz_(&i_mm, &i_jm, &i_isw, i_s, o_wz, w_q);
+    shtswz_(&i_mm, &i_jm, &i_isw, i_s, o_wz, i_work);
 
     {int array_shape[1] = {2*i_jm+1};
      wz = dcl_crealary2obj(o_wz, (2*i_jm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return wz;
 
 }
 
 static VALUE
-dcl_shtswm(obj, mm, jm, m, isw, s)
-    VALUE obj, mm, jm, m, isw, s;
+dcl_shtswm(obj, mm, jm, m, isw, s, work)
+    VALUE obj, mm, jm, m, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -497,7 +545,7 @@ dcl_shtswm(obj, mm, jm, m, isw, s)
     real *i_s;
     real *o_wr;
     real *o_wi;
-    real *w_q;
+    real *i_work;
     VALUE wr;
     VALUE wi;
 
@@ -520,18 +568,25 @@ dcl_shtswm(obj, mm, jm, m, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_m = NUM2INT(m);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_wr= ALLOCA_N(real, (2*i_jm+1));
     o_wi= ALLOCA_N(real, (2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtswm_(&i_mm, &i_jm, &i_m, &i_isw, i_s, o_wr, o_wi, w_q);
+    shtswm_(&i_mm, &i_jm, &i_m, &i_isw, i_s, o_wr, o_wi, i_work);
 
     {int array_shape[1] = {2*i_jm+1};
      wr = dcl_crealary2obj(o_wr, (2*i_jm+1), 1, array_shape);
@@ -541,14 +596,15 @@ dcl_shtswm(obj, mm, jm, m, isw, s)
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(2, wr, wi);
 
 }
 
 static VALUE
-dcl_shtswj(obj, mm, jm, isw, j, m1, m2, s)
-    VALUE obj, mm, jm, isw, j, m1, m2, s;
+dcl_shtswj(obj, mm, jm, isw, j, m1, m2, s, work)
+    VALUE obj, mm, jm, isw, j, m1, m2, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -558,7 +614,7 @@ dcl_shtswj(obj, mm, jm, isw, j, m1, m2, s)
     integer i_m2;
     real *i_s;
     real *o_wj;
-    real *w_q;
+    real *i_work;
     VALUE wj;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -586,6 +642,13 @@ dcl_shtswj(obj, mm, jm, isw, j, m1, m2, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -594,32 +657,33 @@ dcl_shtswj(obj, mm, jm, isw, j, m1, m2, s)
     i_m1 = NUM2INT(m1);
     i_m2 = NUM2INT(m2);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_wj= ALLOCA_N(real, (2*i_mm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtswj_(&i_mm, &i_jm, &i_isw, &i_j, &i_m1, &i_m2, i_s, o_wj, w_q);
+    shtswj_(&i_mm, &i_jm, &i_isw, &i_j, &i_m1, &i_m2, i_s, o_wj, i_work);
 
     {int array_shape[1] = {2*i_mm+1};
      wj = dcl_crealary2obj(o_wj, (2*i_mm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return wj;
 
 }
 
 static VALUE
-dcl_shtw2s(obj, mm, jm, isw, s)
-    VALUE obj, mm, jm, isw, s;
+dcl_shtw2s(obj, mm, jm, isw, s, work)
+    VALUE obj, mm, jm, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
     integer i_isw;
     real *i_s;
     real *o_w;
-    real *w_q;
+    real *i_work;
     VALUE w;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -638,37 +702,45 @@ dcl_shtw2s(obj, mm, jm, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (i_mm+1)*(i_mm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*DFLT_SIZE+15);
 
-    shtw2s_(&i_mm, &i_jm, &i_isw, i_s, o_w, w_q);
+    shtw2s_(&i_mm, &i_jm, &i_isw, i_s, o_w, i_work);
 
     {int array_shape[1] = {(i_mm+1)*(i_mm+1)};
      w = dcl_crealary2obj(o_w, (i_mm+1)*(i_mm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return w;
 
 }
 
 static VALUE
-dcl_shtw2g(obj, mm, jm, im, w)
-    VALUE obj, mm, jm, im, w;
+dcl_shtw2g(obj, mm, jm, im, w, work)
+    VALUE obj, mm, jm, im, w, work;
 {
     integer i_mm;
     integer i_jm;
     integer i_im;
     real *i_w;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE g;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -687,30 +759,38 @@ dcl_shtw2g(obj, mm, jm, im, w)
            (rb_obj_is_kind_of(w, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_im = NUM2INT(im);
     i_w = dcl_obj2crealary(w);
+    i_work = dcl_obj2crealary(work);
 
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtw2g_(&i_mm, &i_jm, &i_im, i_w, o_g, w_q);
+    shtw2g_(&i_mm, &i_jm, &i_im, i_w, o_g, i_work);
 
     {int array_shape[1] = {(2*i_im+1)*(2*i_jm+1)};
      g = dcl_crealary2obj(o_g, (2*i_im+1)*(2*i_jm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_w);
+    dcl_freecrealary(i_work);
 
     return g;
 
 }
 
 static VALUE
-dcl_shtwga(obj, mm, jm, im, m1, m2, w)
-    VALUE obj, mm, jm, im, m1, m2, w;
+dcl_shtwga(obj, mm, jm, im, m1, m2, w, work)
+    VALUE obj, mm, jm, im, m1, m2, w, work;
 {
     integer i_mm;
     integer i_jm;
@@ -719,7 +799,7 @@ dcl_shtwga(obj, mm, jm, im, m1, m2, w)
     integer i_m2;
     real *i_w;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE g;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -744,6 +824,13 @@ dcl_shtwga(obj, mm, jm, im, m1, m2, w)
            (rb_obj_is_kind_of(w, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -751,25 +838,26 @@ dcl_shtwga(obj, mm, jm, im, m1, m2, w)
     i_m1 = NUM2INT(m1);
     i_m2 = NUM2INT(m2);
     i_w = dcl_obj2crealary(w);
+    i_work = dcl_obj2crealary(work);
 
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtwga_(&i_mm, &i_jm, &i_im, &i_m1, &i_m2, i_w, o_g, w_q);
+    shtwga_(&i_mm, &i_jm, &i_im, &i_m1, &i_m2, i_w, o_g, i_work);
 
     {int array_shape[1] = {(2*i_im+1)*(2*i_jm+1)};
      g = dcl_crealary2obj(o_g, (2*i_im+1)*(2*i_jm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_w);
+    dcl_freecrealary(i_work);
 
     return g;
 
 }
 
 static VALUE
-dcl_shtwgm(obj, mm, jm, im, m, wr, wi)
-    VALUE obj, mm, jm, im, m, wr, wi;
+dcl_shtwgm(obj, mm, jm, im, m, wr, wi, work)
+    VALUE obj, mm, jm, im, m, wr, wi, work;
 {
     integer i_mm;
     integer i_jm;
@@ -778,7 +866,7 @@ dcl_shtwgm(obj, mm, jm, im, m, wr, wi)
     real *i_wr;
     real *i_wi;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE g;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -807,6 +895,13 @@ dcl_shtwgm(obj, mm, jm, im, m, wr, wi)
            (rb_obj_is_kind_of(wi, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -814,11 +909,11 @@ dcl_shtwgm(obj, mm, jm, im, m, wr, wi)
     i_m = NUM2INT(m);
     i_wr = dcl_obj2crealary(wr);
     i_wi = dcl_obj2crealary(wi);
+    i_work = dcl_obj2crealary(work);
 
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtwgm_(&i_mm, &i_jm, &i_im, &i_m, i_wr, i_wi, o_g, w_q);
+    shtwgm_(&i_mm, &i_jm, &i_im, &i_m, i_wr, i_wi, o_g, i_work);
 
     {int array_shape[1] = {(2*i_im+1)*(2*i_jm+1)};
      g = dcl_crealary2obj(o_g, (2*i_im+1)*(2*i_jm+1), 1, array_shape);
@@ -826,6 +921,7 @@ dcl_shtwgm(obj, mm, jm, im, m, wr, wi)
 
     dcl_freecrealary(i_wr);
     dcl_freecrealary(i_wi);
+    dcl_freecrealary(i_work);
 
     return g;
 
@@ -874,8 +970,8 @@ dcl_shtwgz(obj, jm, im, wz)
 }
 
 static VALUE
-dcl_shtwgj(obj, mm, im, m1, m2, wj)
-    VALUE obj, mm, im, m1, m2, wj;
+dcl_shtwgj(obj, mm, im, m1, m2, wj, work)
+    VALUE obj, mm, im, m1, m2, wj, work;
 {
     integer i_mm;
     integer i_im;
@@ -883,7 +979,7 @@ dcl_shtwgj(obj, mm, im, m1, m2, wj)
     integer i_m2;
     real *i_wj;
     real *o_gj;
-    real *w_q;
+    real *i_work;
     VALUE gj;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -905,38 +1001,46 @@ dcl_shtwgj(obj, mm, im, m1, m2, wj)
            (rb_obj_is_kind_of(wj, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_im = NUM2INT(im);
     i_m1 = NUM2INT(m1);
     i_m2 = NUM2INT(m2);
     i_wj = dcl_obj2crealary(wj);
+    i_work = dcl_obj2crealary(work);
 
     o_gj= ALLOCA_N(real, (2*i_im+1));
-    w_q= ALLOCA_N(real, (DFLT_SIZE+1)*(4*DFLT_SIZE+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtwgj_(&i_mm, &i_im, &i_m1, &i_m2, i_wj, o_gj, w_q);
+    shtwgj_(&i_mm, &i_im, &i_m1, &i_m2, i_wj, o_gj, i_work);
 
     {int array_shape[1] = {2*i_im+1};
      gj = dcl_crealary2obj(o_gj, (2*i_im+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_wj);
+    dcl_freecrealary(i_work);
 
     return gj;
 
 }
 
 static VALUE
-dcl_shtg2w(obj, mm, jm, im, g)
-    VALUE obj, mm, jm, im, g;
+dcl_shtg2w(obj, mm, jm, im, g, work)
+    VALUE obj, mm, jm, im, g, work;
 {
     integer i_mm;
     integer i_jm;
     integer i_im;
     real *i_g;
     real *o_w;
-    real *w_q;
+    real *i_work;
     VALUE w;
 
     if ((TYPE(mm) != T_BIGNUM) || (TYPE(mm) != T_FIXNUM)) {
@@ -955,30 +1059,38 @@ dcl_shtg2w(obj, mm, jm, im, g)
            (rb_obj_is_kind_of(g, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_im = NUM2INT(im);
     i_g = dcl_obj2crealary(g);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (2*i_jm+1)*(2*i_mm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtg2w_(&i_mm, &i_jm, &i_im, i_g, o_w, w_q);
+    shtg2w_(&i_mm, &i_jm, &i_im, i_g, o_w, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(2*i_mm+1)};
      w = dcl_crealary2obj(o_w, (2*i_jm+1)*(2*i_mm+1), 1, array_shape);
     }
 
     dcl_freecrealary(i_g);
+    dcl_freecrealary(i_work);
 
     return w;
 
 }
 
 static VALUE
-dcl_shts2g(obj, mm, jm, im, isw, s)
-    VALUE obj, mm, jm, im, isw, s;
+dcl_shts2g(obj, mm, jm, im, isw, s, work)
+    VALUE obj, mm, jm, im, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -987,7 +1099,7 @@ dcl_shts2g(obj, mm, jm, im, isw, s)
     real *i_s;
     real *o_w;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE w;
     VALUE g;
 
@@ -1010,18 +1122,25 @@ dcl_shts2g(obj, mm, jm, im, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_im = NUM2INT(im);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (2*i_jm+1)*(2*i_mm+1));
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shts2g_(&i_mm, &i_jm, &i_im, &i_isw, i_s, o_w, o_g, w_q);
+    shts2g_(&i_mm, &i_jm, &i_im, &i_isw, i_s, o_w, o_g, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(2*i_mm+1)};
      w = dcl_crealary2obj(o_w, (2*i_jm+1)*(2*i_mm+1), 1, array_shape);
@@ -1031,14 +1150,15 @@ dcl_shts2g(obj, mm, jm, im, isw, s)
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(2, w, g);
 
 }
 
 static VALUE
-dcl_shtsga(obj, mm, jm, im, isw, m1, m2, s)
-    VALUE obj, mm, jm, im, isw, m1, m2, s;
+dcl_shtsga(obj, mm, jm, im, isw, m1, m2, s, work)
+    VALUE obj, mm, jm, im, isw, m1, m2, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -1049,7 +1169,7 @@ dcl_shtsga(obj, mm, jm, im, isw, m1, m2, s)
     real *i_s;
     real *o_w;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE w;
     VALUE g;
 
@@ -1078,6 +1198,13 @@ dcl_shtsga(obj, mm, jm, im, isw, m1, m2, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -1086,12 +1213,12 @@ dcl_shtsga(obj, mm, jm, im, isw, m1, m2, s)
     i_m1 = NUM2INT(m1);
     i_m2 = NUM2INT(m2);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (2*i_jm+1)*(2*i_mm+1));
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtsga_(&i_mm, &i_jm, &i_im, &i_isw, &i_m1, &i_m2, i_s, o_w, o_g, w_q);
+    shtsga_(&i_mm, &i_jm, &i_im, &i_isw, &i_m1, &i_m2, i_s, o_w, o_g, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(2*i_mm+1)};
      w = dcl_crealary2obj(o_w, (2*i_jm+1)*(2*i_mm+1), 1, array_shape);
@@ -1101,14 +1228,15 @@ dcl_shtsga(obj, mm, jm, im, isw, m1, m2, s)
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(2, w, g);
 
 }
 
 static VALUE
-dcl_shtsgz(obj, mm, jm, im, isw, s)
-    VALUE obj, mm, jm, im, isw, s;
+dcl_shtsgz(obj, mm, jm, im, isw, s, work)
+    VALUE obj, mm, jm, im, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -1117,7 +1245,7 @@ dcl_shtsgz(obj, mm, jm, im, isw, s)
     real *i_s;
     real *o_wz;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE wz;
     VALUE g;
 
@@ -1140,18 +1268,25 @@ dcl_shtsgz(obj, mm, jm, im, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_im = NUM2INT(im);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_wz= ALLOCA_N(real, (2*i_jm+1));
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtsgz_(&i_mm, &i_jm, &i_im, &i_isw, i_s, o_wz, o_g, w_q);
+    shtsgz_(&i_mm, &i_jm, &i_im, &i_isw, i_s, o_wz, o_g, i_work);
 
     {int array_shape[1] = {2*i_jm+1};
      wz = dcl_crealary2obj(o_wz, (2*i_jm+1), 1, array_shape);
@@ -1161,14 +1296,15 @@ dcl_shtsgz(obj, mm, jm, im, isw, s)
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(2, wz, g);
 
 }
 
 static VALUE
-dcl_shtsgm(obj, mm, jm, im, m, isw, s)
-    VALUE obj, mm, jm, im, m, isw, s;
+dcl_shtsgm(obj, mm, jm, im, m, isw, s, work)
+    VALUE obj, mm, jm, im, m, isw, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -1179,7 +1315,7 @@ dcl_shtsgm(obj, mm, jm, im, m, isw, s)
     real *o_wr;
     real *o_wi;
     real *o_g;
-    real *w_q;
+    real *i_work;
     VALUE wr;
     VALUE wi;
     VALUE g;
@@ -1206,6 +1342,13 @@ dcl_shtsgm(obj, mm, jm, im, m, isw, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -1213,13 +1356,13 @@ dcl_shtsgm(obj, mm, jm, im, m, isw, s)
     i_m = NUM2INT(m);
     i_isw = NUM2INT(isw);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_wr= ALLOCA_N(real, (2*i_jm+1));
     o_wi= ALLOCA_N(real, (2*i_jm+1));
     o_g= ALLOCA_N(real, (2*i_im+1)*(2*i_jm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtsgm_(&i_mm, &i_jm, &i_im, &i_m, &i_isw, i_s, o_wr, o_wi, o_g, w_q);
+    shtsgm_(&i_mm, &i_jm, &i_im, &i_m, &i_isw, i_s, o_wr, o_wi, o_g, i_work);
 
     {int array_shape[1] = {2*i_jm+1};
      wr = dcl_crealary2obj(o_wr, (2*i_jm+1), 1, array_shape);
@@ -1232,14 +1375,15 @@ dcl_shtsgm(obj, mm, jm, im, m, isw, s)
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(3, wr, wi, g);
 
 }
 
 static VALUE
-dcl_shtsgj(obj, mm, jm, im, isw, j, m1, m2, s)
-    VALUE obj, mm, jm, im, isw, j, m1, m2, s;
+dcl_shtsgj(obj, mm, jm, im, isw, j, m1, m2, s, work)
+    VALUE obj, mm, jm, im, isw, j, m1, m2, s, work;
 {
     integer i_mm;
     integer i_jm;
@@ -1251,7 +1395,7 @@ dcl_shtsgj(obj, mm, jm, im, isw, j, m1, m2, s)
     real *i_s;
     real *o_wj;
     real *o_gj;
-    real *w_q;
+    real *i_work;
     VALUE wj;
     VALUE gj;
 
@@ -1283,6 +1427,13 @@ dcl_shtsgj(obj, mm, jm, im, isw, j, m1, m2, s)
            (rb_obj_is_kind_of(s, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
@@ -1292,12 +1443,12 @@ dcl_shtsgj(obj, mm, jm, im, isw, j, m1, m2, s)
     i_m1 = NUM2INT(m1);
     i_m2 = NUM2INT(m2);
     i_s = dcl_obj2crealary(s);
+    i_work = dcl_obj2crealary(work);
 
     o_wj= ALLOCA_N(real, (2*i_mm+1));
     o_gj= ALLOCA_N(real, (2*i_im+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtsgj_(&i_mm, &i_jm, &i_im, &i_isw, &i_j, &i_m1, &i_m2, i_s, o_wj, o_gj, w_q);
+    shtsgj_(&i_mm, &i_jm, &i_im, &i_isw, &i_j, &i_m1, &i_m2, i_s, o_wj, o_gj, i_work);
 
     {int array_shape[1] = {2*i_mm+1};
      wj = dcl_crealary2obj(o_wj, (2*i_mm+1), 1, array_shape);
@@ -1307,14 +1458,15 @@ dcl_shtsgj(obj, mm, jm, im, isw, j, m1, m2, s)
     }
 
     dcl_freecrealary(i_s);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(2, wj, gj);
 
 }
 
 static VALUE
-dcl_shtg2s(obj, mm, jm, im, isw, g)
-    VALUE obj, mm, jm, im, isw, g;
+dcl_shtg2s(obj, mm, jm, im, isw, g, work)
+    VALUE obj, mm, jm, im, isw, g, work;
 {
     integer i_mm;
     integer i_jm;
@@ -1323,7 +1475,7 @@ dcl_shtg2s(obj, mm, jm, im, isw, g)
     real *i_g;
     real *o_w;
     real *o_s;
-    real *w_q;
+    real *i_work;
     VALUE w;
     VALUE s;
 
@@ -1346,18 +1498,25 @@ dcl_shtg2s(obj, mm, jm, im, isw, g)
            (rb_obj_is_kind_of(g, cNArray) != Qtrue)) {
          rb_raise(rb_eTypeError, "invalid type");
        }  -- no check since obj2c*ary will do that */
+    if (TYPE(work) == T_FLOAT) {
+      work = rb_Array(work);
+    }
+    /* if ((TYPE(work) != T_ARRAY) && 
+           (rb_obj_is_kind_of(work, cNArray) != Qtrue)) {
+         rb_raise(rb_eTypeError, "invalid type");
+       }  -- no check since obj2c*ary will do that */
 
     i_mm = NUM2INT(mm);
     i_jm = NUM2INT(jm);
     i_im = NUM2INT(im);
     i_isw = NUM2INT(isw);
     i_g = dcl_obj2crealary(g);
+    i_work = dcl_obj2crealary(work);
 
     o_w= ALLOCA_N(real, (2*i_jm+1)*(2*i_mm+1));
     o_s= ALLOCA_N(real, (i_mm+1)*(i_mm+1));
-    w_q= ALLOCA_N(real, (i_jm+1)*(4*i_jm+5*i_mm+14)+(i_mm+1)*(i_mm+1)+i_mm+2+6*i_im+15);
 
-    shtg2s_(&i_mm, &i_jm, &i_im, &i_isw, i_g, o_w, o_s, w_q);
+    shtg2s_(&i_mm, &i_jm, &i_im, &i_isw, i_g, o_w, o_s, i_work);
 
     {int array_shape[1] = {(2*i_jm+1)*(2*i_mm+1)};
      w = dcl_crealary2obj(o_w, (2*i_jm+1)*(2*i_mm+1), 1, array_shape);
@@ -1367,6 +1526,7 @@ dcl_shtg2s(obj, mm, jm, im, isw, g)
     }
 
     dcl_freecrealary(i_g);
+    dcl_freecrealary(i_work);
 
     return rb_ary_new3(2, w, s);
 
@@ -1379,25 +1539,25 @@ VALUE mDCL;
     rb_define_module_function(mDCL, "shtint", dcl_shtint, 3);
     rb_define_module_function(mDCL, "shtlap", dcl_shtlap, 3);
     rb_define_module_function(mDCL, "shtnml", dcl_shtnml, 3);
-    rb_define_module_function(mDCL, "shtfun", dcl_shtfun, 3);
-    rb_define_module_function(mDCL, "shtlfw", dcl_shtlfw, 5);
-    rb_define_module_function(mDCL, "shtlbw", dcl_shtlbw, 5);
-    rb_define_module_function(mDCL, "shts2w", dcl_shts2w, 4);
-    rb_define_module_function(mDCL, "shtswa", dcl_shtswa, 6);
-    rb_define_module_function(mDCL, "shtswz", dcl_shtswz, 4);
-    rb_define_module_function(mDCL, "shtswm", dcl_shtswm, 5);
-    rb_define_module_function(mDCL, "shtswj", dcl_shtswj, 7);
-    rb_define_module_function(mDCL, "shtw2s", dcl_shtw2s, 4);
-    rb_define_module_function(mDCL, "shtw2g", dcl_shtw2g, 4);
-    rb_define_module_function(mDCL, "shtwga", dcl_shtwga, 6);
-    rb_define_module_function(mDCL, "shtwgm", dcl_shtwgm, 6);
+    rb_define_module_function(mDCL, "shtfun", dcl_shtfun, 4);
+    rb_define_module_function(mDCL, "shtlfw", dcl_shtlfw, 6);
+    rb_define_module_function(mDCL, "shtlbw", dcl_shtlbw, 6);
+    rb_define_module_function(mDCL, "shts2w", dcl_shts2w, 5);
+    rb_define_module_function(mDCL, "shtswa", dcl_shtswa, 7);
+    rb_define_module_function(mDCL, "shtswz", dcl_shtswz, 5);
+    rb_define_module_function(mDCL, "shtswm", dcl_shtswm, 6);
+    rb_define_module_function(mDCL, "shtswj", dcl_shtswj, 8);
+    rb_define_module_function(mDCL, "shtw2s", dcl_shtw2s, 5);
+    rb_define_module_function(mDCL, "shtw2g", dcl_shtw2g, 5);
+    rb_define_module_function(mDCL, "shtwga", dcl_shtwga, 7);
+    rb_define_module_function(mDCL, "shtwgm", dcl_shtwgm, 7);
     rb_define_module_function(mDCL, "shtwgz", dcl_shtwgz, 3);
-    rb_define_module_function(mDCL, "shtwgj", dcl_shtwgj, 5);
-    rb_define_module_function(mDCL, "shtg2w", dcl_shtg2w, 4);
-    rb_define_module_function(mDCL, "shts2g", dcl_shts2g, 5);
-    rb_define_module_function(mDCL, "shtsga", dcl_shtsga, 7);
-    rb_define_module_function(mDCL, "shtsgz", dcl_shtsgz, 5);
-    rb_define_module_function(mDCL, "shtsgm", dcl_shtsgm, 6);
-    rb_define_module_function(mDCL, "shtsgj", dcl_shtsgj, 8);
-    rb_define_module_function(mDCL, "shtg2s", dcl_shtg2s, 5);
+    rb_define_module_function(mDCL, "shtwgj", dcl_shtwgj, 6);
+    rb_define_module_function(mDCL, "shtg2w", dcl_shtg2w, 5);
+    rb_define_module_function(mDCL, "shts2g", dcl_shts2g, 6);
+    rb_define_module_function(mDCL, "shtsga", dcl_shtsga, 8);
+    rb_define_module_function(mDCL, "shtsgz", dcl_shtsgz, 6);
+    rb_define_module_function(mDCL, "shtsgm", dcl_shtsgm, 7);
+    rb_define_module_function(mDCL, "shtsgj", dcl_shtsgj, 9);
+    rb_define_module_function(mDCL, "shtg2s", dcl_shtg2s, 6);
 }
